package graphismes.internes;

import graphismes.externes.Fenetre;
import graphismes.outils.LecteurImage;
import personnages.Sualocine;

import javax.swing.*;
import java.awt.*;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.image.BufferedImage;


public class Fuite extends JPanel implements KeyListener, Runnable {

    private String cheminBackground = "./res/images/BG/BG.png";

    private boolean enLAir = false;
    private BufferedImage background;

    private Map map;
    public static Sualocine sualocine;
    private Score score = new Score();

    private float interpolation;
    private int fps = 60;
    private int frameCount = 0;
    private Thread animator;
    private int v;
    private boolean hautSaut = false;
    private boolean finSaut = false;
    public static long vitesse = 0;
    public static int acceleration = 2;
    private int difficulte = 0;
    public boolean paused = false;

    public Fuite(){
        super();
        init();
        lancerJeu();
    }

    private void init(){
        this.setOpaque(false);
        this.setSize(Fenetre.getFenetre().getSize());
        this.setFocusable(true);
        this.requestFocusInWindow();
        this.addKeyListener(this);
        sualocine = new Sualocine();
        background = LecteurImage.donneImage(cheminBackground);
        map = new Map();

    }

    private void lancerJeu(){
        Thread loop = new Thread()
        {
            public void run()
            {
                gameLoop();
            }
        };
        loop.start();
    }

    private void gameLoop()
    {
        final double GAME_HERTZ = 50.0;
        final double TIME_BETWEEN_UPDATES = 1000000000 / GAME_HERTZ;
        //At the very most we will update the game this many times before a new render.
        //If you're worried about visual hitches more than perfect timing, set this to 1.
        final int MAX_UPDATES_BEFORE_RENDER = 1;

        double lastUpdateTime = System.nanoTime();
        double lastRenderTime = System.nanoTime();

        //If we are able to get as high as this FPS, don't render again.
        final double TARGET_FPS = 80;
        final double TARGET_TIME_BETWEEN_RENDERS = 1000000000 / TARGET_FPS;

        int lastSecondTime = (int) (lastUpdateTime / 1000000000);





        double now = System.nanoTime();
        int updateCount = 0;
        while (sualocine.estVivante())
        {
            now = System.nanoTime();
            updateCount = 0;

            if (!paused)
            {
                //Do as many game updates as we need to, potentially playing catchup.
                while( now - lastUpdateTime > TIME_BETWEEN_UPDATES && updateCount < MAX_UPDATES_BEFORE_RENDER )
                {
                    updateGame();
                    lastUpdateTime += TIME_BETWEEN_UPDATES;
                    updateCount++;
                }

                //If for some reason an update takes forever, we don't want to do an insane number of catchups.
                //If you were doing some sort of game that needed to keep EXACT time, you would get rid of this.
                if ( now - lastUpdateTime > TIME_BETWEEN_UPDATES)
                {
                    lastUpdateTime = now - TIME_BETWEEN_UPDATES;
                }

                //Render. To do so, we need to calculate interpolation for a smooth render.
                float interpolation = Math.min(1.0f, (float) ((now - lastUpdateTime) / TIME_BETWEEN_UPDATES) );
                drawGame(interpolation);
                lastRenderTime = now;

                //Update the frames we got.
                int thisSecond = (int) (lastUpdateTime / 1000000000);
                if (thisSecond > lastSecondTime)
                {
                    //System.out.println("NEW SECOND " + thisSecond + " " + frameCount);

                    difficulte++;
                    if(difficulte%5 == 0) {
                        acceleration++;
                        difficulte = 0;
                    }
                    score.incremente();
                    fps = frameCount;
                    frameCount = 0;
                    lastSecondTime = thisSecond;
                }

                //Yield until it has been at least the target time between renders. This saves the CPU from hogging.
                while ( now - lastRenderTime < TARGET_TIME_BETWEEN_RENDERS && now - lastUpdateTime < TIME_BETWEEN_UPDATES)
                {
                    Thread.yield();

                    //This stops the app from consuming all your CPU. It makes this slightly less accurate, but is worth it.
                    //You can remove this line and it will still work (better), your CPU just climbs on certain OSes.
                    try {Thread.sleep(1);} catch(Exception e) {}

                    now = System.nanoTime();
                }

                vitesse += acceleration;
            }
        }
    }


    private void drawGame(float interpolation)
    {
        this.setInterpolation(interpolation);
        this.repaint();
    }

    public void setInterpolation(float interp)
    {
        interpolation = interp;
    }

    //Fonction qui calcule tout dans le jeu
    private void updateGame(){
        sualocine.move();
        if(sualocine.getDY() == 1 && enLAir == false){
            enLAir = true;
            animator = new Thread(this);
            animator.start();
        }
        map.update();

        System.out.println(sualocine.collisionCarre.testCollision(map.collisionMap));
    }

    //Fonction qui affiche jeu
    public void paintComponent(Graphics g)
    {
        Graphics2D g2d = (Graphics2D)g;
        //on affiche d'abord le fond d'écran
        g2d.drawImage(background, 0, 0, null);
        //on affiche la map
        map.setG2D(g2d);
        map.paint();
        //on affiche le personnage
        g2d.drawImage(sualocine.getImage(), 150, v,sualocine.getHAUTEUR(), sualocine.getLARGEUR(), null);;

        //on affiche le score.
        score.paint(g2d);

        //on compte les frames.
        frameCount++;
    }



    @Override
    public void keyTyped(KeyEvent keyEvent) {} //lui on s'en fout

    @Override
    public void keyReleased(KeyEvent e) {
        sualocine.keyReleased(e);
    }

    @Override
    public void keyPressed(KeyEvent e) {
        sualocine.keyPressed(e);

        int touche = e.getKeyCode();

        if(touche == KeyEvent.VK_ESCAPE){
            System.exit(0);
        }
        if(touche == KeyEvent.VK_SPACE){
           if(paused)
               paused = false;
           else
               paused = true;
        }
    }

    public void run(){
        long beforeTime, timeDiff, sleep;
        beforeTime = System.currentTimeMillis();
        v=sualocine.getY();
        while(finSaut == false ){
            cycle();
            timeDiff = System.currentTimeMillis() - beforeTime;
            sleep = 10 - timeDiff;
            if (sleep < 0)
                sleep = 2;
            try{
                Thread.sleep(sleep);
            }catch (Exception e){
                e.printStackTrace();
            }
            beforeTime = System.currentTimeMillis();
        }
        finSaut = false;
        hautSaut = false;
        enLAir = false;
    }



    public void cycle(){
        if(hautSaut == false)
            v-=2;
        if( v < sualocine.getY() - 250)
            hautSaut = true;
        if( hautSaut == true &&  v <= sualocine.getY())
            v+=2;


        if( v >= sualocine.getY() ) {
            finSaut = true;
            //v=500;
        }
    }
}




